      Program Layr2Plt

! Convert layer-cake model (in terms of layer thicknesses or d2bot)
! into a piecewise continuous velocity function for plotting purposes

! Uses a control file to obtain information regarding the format
! of the input file.  Here is an example:

!!Control file for layr2plt
!! Revision of program involving a change in the control file on this date:
!   12/12/21
!!Was the input file produced by program Site_Amp? (y/n):
!! If Y, then the remaining input parameters down to
!! the character string for the output file name are
!! ignored, although dummy values need to be included
!! in the control file
! N
!!Number of header lines to skip:
! 4
!!Read in density also? (y/n): 
! N
!!NColumns, column numbers containing depth (or thickness), velocity, 
!! and density (if used):
! 2 1 2 
!!Depth to top (0), bottom (1) or layer thickness (2)?
! If depth to top, the last entry in the model file will be the depth to bottom of the 
! last layer, but an associated bogus velocity must be entered (e.g., -999).
!  2
!!If layer thickness, and the thickness=0, the velocity is assumed to be a halfspace velocity.
!!For plotting purposes, the next entry is a pseudo-layer thickness for the halfspace.
!!A placeholder is needed even if not used.
! 1.0
!!Enter units of velocity (kps, mps) (depth units are assumed to be consistent
!!with the velocity units---km for kps and m for mps):
!   kps
!!Character string to add to end of input file name:
! .ss.col
!!List of input files
!bj97_hr_vs30_1100_30_lyrs_pl1_2_3.mdl
!stop 
  
! The input file can also be from program Site_Amp

! Dates: 12/18/95 - Written by D. Boore... as a special purpose
!                   program, reading the rattle.in format
!                   created by F4Rattle.For
!        01/25/95 - Renamed Layr2Plt, because the name Layr2Cnt implied
!                   a layer model was being converted into a continuous model,
!                   which it is, but with step changes.  The smoothly varying
!                   velocity model used in the generic rock study was converted
!                   into a constant-velocity layered model in Site_Amp.For,
!                   and this was converted into "continuous" model that could
!                   be used in CoPlot.
!        01/14/97 - Obtained name of input and output file from user; loop over
!                   obtaining names.  Commented out code regarding rattle
!                   including specialization to read rattle input files.
!                   Add personalized column headings.  As a default,
!                   constuct output file name automatically from input 
!                   file name.
!        01/05/98 - Allow for zero velocity layer, and use eof to mark the 
!                   last line.  I also improved the construction of the
!                   column header and I write the input velocity line to the
!                   screen.  In addition, I allow for a text header to the 
!                   input file to allow for explanations.
!        04/23/98 - Improved dealing with the number of header 
!                   lines before the data in the input file.  Now the program
!                   prints out the first three lines and asks on which line
!                   the data start.  The program then rewinds the file and
!                   proceeds accordingly.  This is more general than before
!                   and allows for 1 or 2 header lines.
!        06/17/00 - Added capability of reading a file made by program 
!                   Site_Amp.  Also, suggest "COL" as extension of output file.
!        10/12/00 - Eradicate a few bugs
!        02/10/03 - Use trim_c to strip leading and trailing blanks from
!                   file names (modern compilers allow blanks in file names,
!                   and this might produce errors if the file name obtained 
!                   from a control file is used as is--- for example, if
!                   the name in the control file is preceeded by blanks).
!        04/30/04 - Compile using LF95
!        08/19/05 - Add "layered_model" before stem name (because CoPlot
!                   deletes the extension when importing the file, so files
!                   made using both site_amp and layr2plt will have the same 
!                   dt file name).
!        05/23/06 - Include option to ignore reading density from model file
!        05/23/06 - Allow depths to be depth-to-bottom or thickness of each layer
!        05/25/06 - Add slowness to output
!        09/06/06 - Use a control file to control the format of the input file,
!                   thus making the program more versatile.
!        09/09/06 - Corrected small error in writing column labels
!        03/10/12 - Add name of model file to output
!                 - Add date to control file
!                 - Ask if units are mps or kps, and write output with both units
!                 - If read in, write density as column 7
!        07/27/12 - Account for modification in site_amp output
!        08/04/12 - Increase resolution of velocity output (this is necessary if
!                    fine depth intervals are used with relatively large velocities in
!                    m/s). 
!                 - Increase length of output name
!                 - Take into account the new output format of site_amp_batch 
!                   and site_amp (es11.5)
!        08/05/12 - Read site_amp input into dummy array so that I can use an unformatted
!                   read (so future revisions of site_amp that involve changes to
!                   output file column widths will not cause errors).
!        02/11/16 - Add depth to top as an option, and use "0" to indicate this ("0" used 
!                   to designate depth to bottom).
!        07/28/17 - If the input file is from site_amp_batch, account for new version of
!                   site_amp_batch output (ncluding reading of MPS or KPS from the output file).
!        07/29/17 - Add qinverse to output
!        12/12/21 - Add thickness of haflspace pseudo layer if model is in terms of layer thickness
!                   and the last entry has a thickness of 0.0.  This option is included if
!                   a portion of the halfspace is to be plotted.
!        02/07/24 - Redo reading site_amp_batch output.  I had used a format free read statement, but it failed
!                   when the input model had fewer layers than the equivalent cvl model. In that case, the input model 
!                   values were represented with "NA".

      real d2t(0:10000), d2b(0:10000), depth(10000), 
     :     vel(10000), dens(10000), qi(10000)
      real thick(10000), vel_in(10000), dens_in(10000), qi_in(10000),
     :     dum(20)
      
      integer ios, ios_model_vals

      character f_ctl*200, f_in*200, f_out*220, temp_c*8, 
     : reply_1*3, reply_rho*3, reply_thick*3,
     : col1name*17, col2name*17, col3name*17,   
     : col4name*17, col5name*17, col6name*17,   
     : col7name*17, col8name*17, 
     : reply*3, bffr*80, buf*100, ext*100, buf4*4
      character cmnts2skip(50)*80, units_c*10

      character version_ctl*8, version_in*30

      logical f_exist, site_amp_output, read_density

      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file ("Enter" = layr2plt.ctl; '//
     :                                 'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'layr2plt.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

!Check version of control file
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      version_in = ' '
      read(nu_ctl,'(a)') version_in
      call trim_c(version_in,nc_version_in)
      
      version_ctl = ' '
      version_ctl = '12/12/21'
      call trim_c(version_ctl,nc_version_ctl)

      if (version_ctl(1:nc_version_ctl) .ne. 
     :    version_in(1:nc_version_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong version date; STOP!'
        close(nu_ctl)
        stop
      end if
      
 
! Read input parameters from control file:

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         site_amp_output = .true.
      else
         site_amp_output = .false.
      end if

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) nheaders2skip

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         read_density = .true.
      else
         read_density = .false.
      end if

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      if (read_density) then
        read(nu_ctl, *) ncol, icol_dh, icol_vel, icol_dens
      else
        read(nu_ctl, *) ncol, icol_dh, icol_vel
      end if

! Depth to top, bottom or thickness?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) id2b_h

! Depth to top, bottom or thickness?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) pseudolayer_thickness

!Specify units of velocity (kps, mps):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      units_c = ' '
      read(nu_ctl,'(a)') units_c
      call trim_c(units_c, nc_units_c)
      call upstr(units_c)
                         ! if depths and velocities are in km, km/s, they will
                         ! be converted to m, m/s
      
! Get extension for output file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, '(a)') ext
      call trim_c(ext,nc_ext)
 
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

      loop over velocity models: DO

        buf = ' '
        read(nu_ctl,'(a)',iostat=ios) buf
        if (ios /= 0) then
          exit loop over velocity models
        end if
        call trim_c(buf, nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if (buf4 == 'STOP') then
          exit loop over velocity models
        end if     
 
        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)
        f_exist = .false.
        inquire(file=f_in(1:nc_f_in), exist=f_exist)
        if (.not. f_exist) then
           write(*,'(a)') 
     :  ' ******* FILE '//f_in(1:nc_f_in)//' DOES NOT EXIST ******* '
           write(*,'(a)') ' Skipping to next file'
          cycle loop over velocity models
        end if
 

        call get_lun(nu_in)
        open(nu_in,file=f_in(1:nc_f_in),status='unknown')

! Construct name of output file and open the file:
        call get_lun(nu_out)
        f_out = ' '
        f_out = f_in(1:nc_f_in)//ext(1:nc_ext)
        call trim_c(f_out,nc_f_out)
      
        open(nu_out,file= f_out(1:nc_f_out),status='unknown')
      
        write(nu_out,'(a)') '! f_ctl = '//f_ctl(1:nc_f_ctl)
        write(nu_out,'(a)') '! f_in = '//f_in(1:nc_f_in)
      

        qi = 0.0
        qi_in = 0.0
 
        if (site_amp_output) then
        
          d2b(0) = 0.0
          
          read_density = .true.

          call skip(nu_in, 5)
          read(nu_in,'(t34,i4)') nlayers

          read(nu_in,'(t19,a3)') units_c(1:3)
          call trim_c(units_c, nc_units_c)
          call upstr(units_c)

          call skip(nu_in, 1)
          
!  bj97_gen-d  bj97_gen-v  bj97_gen-r  bj97_genqi d2layer_bot   tt2lyrbot    thck_lyr   vel_layer   dens_layr    qi_layer slownss_lyr     ttlayer      avgvel     avgdens      avgaoi  bj97_gen-f  bj97_gen-a  bj97_gen-k  bj97_genak  fmin4model        z(m)      v(m/s)    slowness etaFromFunc SRImodFrmFn  etaFromCSV   SRImodCSV     f/fbot
! 0.00000E+00 2.45000E-01 1.88523E+00 0.00000E+00 6.12500E-04 2.50000E-03 6.12500E-04 2.45000E-01 1.88523E+00 0.00000E+00 4.08163E+00 2.50000E-03 2.45000E-01 1.88523E+00 0.00000E+00 1.00000E+02 4.53997E+00 0.00000E+00 4.53997E+00 9.18591E-02 0.00000E+00 2.45000E+02 4.08163E+00 5.60413E-01 5.45057E+00 5.60687E-01 5.45510E+00 1.08862E+03
! 1.00000E-03 2.45000E-01 1.88523E+00 0.00000E+00 6.13011E-04 2.50208E-03 5.10772E-07 2.45002E-01 1.88509E+00 0.00000E+00 4.08160E+00 2.08477E-06 2.45000E-01 1.88523E+00 0.00000E+00 9.99167E+01 4.53997E+00 0.00000E+00 4.53997E+00 9.18591E-02 1.00000E+00 2.45000E+02 4.08163E+00 5.60413E-01 5.45058E+00 5.60687E-01 5.45510E+00 1.08772E+03
! 7.95000E+00 3.49800E+00 2.71790E+00 0.00000E+00 7.91135E-04 3.22912E-03 8.50530E-07 2.45003E-01 1.88523E+00 0.00000E+00 4.08158E+00 3.47150E-06 2.45000E-01 1.88523E+00 0.00000E+00 7.74204E+01 4.53997E+00 0.00000E+00 4.53997E+00 9.18591E-02 7.95000E+03 3.49800E+03 2.85878E-01 5.60482E-01 5.45171E+00 5.60687E-01 5.45510E+00 8.42818E+02
! 8.00000E+00 3.50000E+00 2.71833E+00 0.00000E+00 7.91987E-04 3.23260E-03 8.52335E-07 2.44997E-01 1.88523E+00 0.00000E+00 4.08168E+00 3.47896E-06 2.45000E-01 1.88523E+00 0.00000E+00 7.73371E+01 4.53997E+00 0.00000E+00 4.53997E+00 9.18591E-02 8.00000E+03 3.50000E+03 2.85714E-01 5.60483E-01 5.45172E+00 5.60687E-01 5.45510E+00 8.41911E+02
!          NA          NA          NA          NA 7.92841E-04 3.23609E-03 8.54197E-07 2.45008E-01 1.88523E+00 0.00000E+00 4.08150E+00 3.48641E-06 2.45000E-01 1.88523E+00 0.00000E+00 7.72538E+01 4.53997E+00 0.00000E+00 4.53997E+00 9.18591E-02          NA          NA          NA 5.60483E-01 5.45173E+00 5.60687E-01 5.45510E+00 8.41003E+02
!          NA          NA          NA          NA 7.93697E-04 3.23958E-03 8.55944E-07 2.44985E-01 1.88523E+00 0.00000E+00 4.08188E+00 3.49386E-06 2.45000E-01 1.88523E+00 0.00000E+00 7.71705E+01 4.53997E+00 0.00000E+00 4.53997E+00 9.18591E-02          NA          NA          NA 5.60483E-01 5.45173E+00 5.60687E-01 5.45510E+00 8.40096E+02
!          NA          NA          NA          NA 7.94555E-04 3.24308E-03 8.57923E-07 2.45013E-01 1.88523E+00 0.00000E+00 4.08142E+00 3.50154E-06 2.45000E-01 1.88523E+00 0.00000E+00 7.70872E+01 4.53997E+00 0.00000E+00 4.53997E+00 9.18591E-02          NA          NA          NA 5.60484E-01 5.45174E+00 5.60687E-01 5.45510E+00 8.39189E+02

          do i = 1, nlayers
            read(nu_in,'(t73,4(e12.5))') 
     :         thick(i), vel_in(i), dens_in(i), qi_in(i)
            write(*, '(4(1x,es11.5))') 
     :      thick(i), vel_in(i), dens_in(i), qi_in(i)
            d2b(i) = d2b(i-1) + thick(i)
          end do
        
!          do i = 1, nlayers
!            read(nu_in,*) (dum(j), j=1, 11)
!            thick(i) = dum(7)
!            vel_in(i) = dum(8)
!            dens_in(i) = dum(9)
!            qi_in(i) = dum(11)
!            write(*, '(4(1x,es11.5))') 
!     :      thick(i), vel_in(i), dens_in(i), qi_in(i)
!            d2b(i) = d2b(i-1) + thick(i)
!          end do
        
        else             ! not site_amp ouptut
 
          call skip(nu_in, nheaders2skip)
        
! initialize arrays with 0.0 (this also takes care of 
! setting d2t(0) and d2b(0) to 0.0, which is what the statement did before 02/11/16).
          d2t  = 0.0
          d2b  = 0.0
          
          nlayers = 0 
          loop over model values: DO
            read(nu_in, *, iostat=ios_model_vals) (dum(i), i=1, ncol)
            if (ios_model_vals /= 0) then
              exit loop over model values
            end if
            if (read_density) then
              adepth = dum(icol_dh)
              vel_in(nlayers+1) = dum(icol_vel)
              dens_in(nlayers+1) = dum(icol_dens)
            else
              adepth = dum(icol_dh)
              vel_in(nlayers+1) = dum(icol_vel)
              dens_in(nlayers+1) = 0.0
            end if
        
            if (id2b_h == 0) then    ! entry is depth to top
              d2t(nlayers+1) = adepth
            else if (id2b_h == 1) then    ! entry is depth to bottom
              d2b(nlayers+1) = adepth
            else if (id2b_h == 2) then    ! entry is thickness
              if (adepth == 0.0) then ! assume this is the halfspace
                adepth = pseudolayer_thickness ! add another layer in the halfspace
              end if
              d2b(nlayers+1) = d2b(nlayers) + adepth
            else
              print *,' ERROR: id2b_h is not 0, 1, or 2; QUITTING'
              stop
            end if
        
!            write(*, *) 
!     :    adepth, d2b(nlayers+1), vel_in(nlayers+1), dens_in(nlayers+1)
            nlayers = nlayers + 1
        
          END DO loop over model values

! If depth to top, convert to depth to bottom:
          if (id2b_h == 0) then   
            nlayers = nlayers - 1
            do i = 1, nlayers
              d2b(i) = d2t(i+1)
            end do
          end if          

        end if  ! site_amp output or not

        close(unit=nu_in)

! Convert stack of constant-velocity layers into a velocity-depth function (with
! repeated depths at interfaces) so that it is in the same format as a 
! piecewise continuous function.

        idepth = 0
        do ilayer = 1, nlayers
! now assign array values for the interfaces above and below each layer:
!   first above:
          idepth = idepth + 1
          depth(idepth) = d2b(ilayer-1)
          vel(idepth) = vel_in(ilayer)
          dens(idepth) = dens_in(ilayer)           
          qi(idepth) = qi_in(ilayer)           
!   now below:
          idepth = idepth + 1
          depth(idepth) = d2b(ilayer)
          vel(idepth) = vel_in(ilayer)
          dens(idepth) = dens_in(ilayer)           
          qi(idepth) = qi_in(ilayer)           
        end do
        ndepths = idepth

!* Read idtag:
!        idtag = ' '
!        read(nu_in, '(1x,a)') idtag
!        call trim_c(idtag, nc_id)
!* Set up the column headings:
!        colhead = ' '
!        colhead = idtag(1:nc_id)//'......' ! nc_id + 6 > length of colhead

        col1name = '    D(m)-________'
        col2name = ' Vs(m/s)-________'
        col3name = ' S(ms/m)-________'

        col4name = '   D(km)-________'
        col5name = 'Vs(km/s)-________'
        col6name = ' S(s/km)-________'
        
        col7name = '       R-________'
        col8name = '      QI-________'

        temp_c = f_in(1:index(f_in,'.')-1)
        call trim_c(temp_c, nc)
        if (nc >= 8 ) then
          nc = 8
        end if
        col1name(18-nc:17) = temp_c 
        col2name(18-nc:17) = temp_c 
        col3name(18-nc:17) = temp_c 
        col4name(18-nc:17) = temp_c 
        col5name(18-nc:17) = temp_c 
        col6name(18-nc:17) = temp_c 
        col7name(18-nc:17) = temp_c 
        col8name(18-nc:17) = temp_c 
 
        if (read_density) then
          write(nu_out,'(
     :        1x,a, 1x,a, 1x,a, 
     :        1x,a, 1x,a, 1x,a,
     :        1x,a, 1x,a
     :                                       )') 
     :      col1name, col2name, col3name, 
     :      col4name, col5name, col6name,
     :      col7name, col8name
        else
          write(nu_out,'(
     :        1x,a, 1x,a, 1x,a, 
     :        1x,a, 1x,a, 1x,a, 1x,a 
     :                                       )') 
     :      col1name, col2name, col3name, 
     :      col4name, col5name, col6name, col8name 
        end if
        
        do i = 1, ndepths
          
          if (units_c(1:3) == 'MPS') then
              z_m = depth(i)
              v_mps = vel(i)
              s_mspm = 1000.0/v_mps
              z_km = z_m/1000.0
              v_kps = v_mps/1000.0
              s_spkm = 1.0/v_kps
          else
              z_km = depth(i)
              v_kps = vel(i)
              s_spkm = 1/v_kps
              z_m = 1000.0*z_km
              v_mps = 1000.0*v_kps
              s_mspm = 1000.0/v_mps
          end if
            
          if (read_density) then
            write(nu_out, '(8(7x,es11.5))') 
     :          z_m, v_mps, s_mspm,
     :          z_km, v_kps, s_spkm,
     :          dens(i), qi(i)
          else
            write(nu_out, '(7(7x,es11.5))') 
     :          z_m, v_mps, s_mspm,
     :          z_km, v_kps, s_spkm,
     :          qi(i)
          end if
          
        end do
        
        close(unit=nu_out)

      END DO loop over velocity models
        
      close(nu_ctl)


      end

 
      include 'site_amp_util_subs.for'
      