      Program vel2cvl_model

! Computes an equivalent constant velocity model for a velocity model 

! Note that if a PS suspension log, the velocity model starts from the surface,
! either assuming that the first velocity in the model is the velocity of the
! "missing" layer between the surface and the depth of the first value
! (more precisely, the depth from the surface to the depth of the first PS
! value minus half of the depth interval between the first and second PS
! values), or using a velocity from the control file for the velocity
! of the first layer.  For a PS log model, the number of layers equals
! the number of input values plus one.

! Reads info from a control file; here is an example:

!!Control file for vel2cvl_model
!! Revision of program involving a change in the control file on this date:
!   09/16/21
!!Name of summary file:
!   vel2cvl_model.sum
!!Format of input files: 1= site_amp_batch output, 2 = other
!! Note: if svel, then many of the following parameters are not needed,
!! but placeholder entries are still needed.
! 2
!!Are the velocities from a suspension log survey?
!!If yes, assume that the velocity value is at the midpoint of the layer
! N
!!Velocity for first layer (0 = use the first velocity of the input)
!! Use this for sensitivity studies of vavg for PS log models that do not
!! extend to the surface.   
!   0.0
!!Depth to bottom (d2b) or layer thickness (thk) (note: a value of 0.0 for
!!the first depth from the model file will signal that the model is a
!!piecewise continuous model)?
!!  thk
! d2b
!!Lines to skip to reach data (<0, use skipcmnt; 0, skip none; > 0, specify)
!   +1
!!Ncolumns, column numbers for depth or thickness (allows for the first column
!!being an index) and the velocity 
!   2 1 2
!!Enter units of velocity (kps, mps) (depth units are assumed to be consistent
!!with the velocity units---km for kps and m for mps):
!!   mps
! kps
!!depths from log-spaced sequence (1), linear-spaced sequence (2), individual depths (3), depths for equal travel times(4)?
!! if 4, then the number of travel time subdivisions is given by the first number in the next set of parameters and the
!! number of elements in the temporary travel time array (used to find the layer depths via interpolation, rather than
!! inversion...a kludge, but easier to program) is given by the second number (only two numbers are read).
! 4
!!if log-spaced: ndepths, depth_start (a value > 0.0; it can be very small), depth_stop:
!!if linear-spaced depths: ndepths, depth_start (if = 0, there will be ndepths equal thickness layers, with thickness = depth_stop/ndepths), depth_stop:
!!if individual depths: ndepths, list of ndepths depths:
!!if equal travel times, number of tt subdivisions, number of tt in temporary array (see above):
! 200 1100 ! if equal travel times, 200 layers with 1100 travel times in temporary array
!!20 0.01 8.0
!!20 0.167 0.447 0.766 1.11 1.475 1.856 2.251 2.658 3.075 3.501 3.936 4.377 4.821 5.268 5.717 6.169 6.624 7.08 7.539 8
!!List of files: pairs, input file followed by output file
!!Make sure that there are no blank entries!
!dmb_interpolate_bj97_gr_vhr_to_760mps.mdl
!bj97gr760.200_equal_tt_divisions.mdl
!stop

! Format of input files:
! headers (preceded by "!" if more than one)
! depths, vel (see control file, can have other columns)

! Note: add "!" to beginning of headers in input files (because can then
! have an arbitrary number)

! VERY IMPORTANT --- Make sure that there are no blank entries; if there are
! the next nonblank will be read for the value that should have been
! in the blank space (so a velocity will be read as a depth, and vice versa).

! Dates:   
! 08/20/21 - Written by D. Boore, based on vel2tt
! 08/21/21 - Add thickness to output
! 09/02/21 - Add tt in layer to output
! 09/04/21 - Generalize specification of depths (log- or linear-spaced, or individual depths)
! 09/16/21 - Add option of finding depths for equal travel time increments,
!            based on code_to_compute_depths_for_tt_increments.R
! 10/02/21 - Updated some of the comments in the source file from "svel" to "site_amp output" (they were for the
!            program vel2tt, from which this program was adapted).
!          - debug case when input velocity model is from a site_amp_batch output file.
! 10/17/21 - Finding depths to layers such that the travel time in each layer is constant is not done precisely,
!            inverting for the depths given tt2bot/nlayers.  Instead, a vector of travel times for a large number
!            of layers is created and interpolation is done to find the layer depths.  The number of elements in the
!            temporary array is specified in the ctl file.  I make use of nlayers and attempt to read from the same line,
!            so I do not need to create a new ctl file (the text comments describe how to specify the quantities).
! 10/22/21 - Start the temporary tt array at z=0, t=0
! 10/23/21 - Changed name from vel2constant_velocity_model to vel2cvl_model ("cvl" = constant velocity layer)
!          - Change units of slowness to s/km or ms/m for velocity units of kps or mps.
!          - If log spaced, need depth_start > 0 to avoid taking the log of 0.0, and the first layer will have a constant velocity from 0.0 to
!            depth_start.  depth_start can be very small, as a way of avoiding log 0.0.
!          - If linear spaced, if depth_start = 0.0, the first layer will be constant velocity from 0.0 to depth_stop/ndepths.
!            This allows for equal depth spaced layers.  If linear spaced but depth_start > 0, the first layer will have a thickness of depth_start.

      real depth_in(0:10000), d2b_in(0:10000), vel_in(0:10000), 
     :     slope_v(0:10000), depth_temp(0:10000), vel_temp(0:10000),
     :     d2b(0:10000), d4tt(0:10000), tt(0:10000),
     :     depth_model(10000), vel_model(10000), 
     :     tt_model(10000), avgvel(10000), vel(10000),depth_out(10000)
      real dum(10), tt4depths(10000), d4interp(10000), tt4interp(10000)
      
      character f_sum*100, f_in*200, f_tt*200, buf1*200, buf2*200,
     :          buf4*4, f_out*200
      character path*200, f_in_with_path*400
      character cmnts2skip(50)*80, type*1, constant_vel*1, 
     :          type_of_depth*10, units_c*10

      character date_ctl_correct*8, date_ctl_in*30

      integer status, vel_file_format

      character f_ctl*40
      logical f_exist, susp_log, extend_depth, specify_density
      logical compute_for_bottom

      pi = 4.0 * atan(1.0)
      d2r = pi/180.0

      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file '//
     :             '(Enter = vel2cvl_model.ctl; '//
     :                                 'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl == ' ') f_ctl = 'vel2cvl_model.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '09/16/21'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong version date; STOP!'
        write(*,'(a)') 
     :     ' The correct date is '//
     :            date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
      
! Get summary file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
      write(nu_sum,'(9x,a, 8x,a, 6x,a, 6x,a, 10x,a, 1x,a)')
     :   'file', 'd2b', 'thk_max', 'd_xtrap_to_30m', 'v30', 'class'

! DEBUG
      call get_lun(nu_debug)
      open(unit=nu_debug,file='debug.out',status='unknown')
! DEBUG

! What file format?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) vel_file_format
      
! DEBUG
      write(nu_debug,'(a,1x,i5)') ' vel_file_format = ',vel_file_format
! DEBUG
      
! Are velocities from a suspension log?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf1 = ' '
      read(nu_ctl,'(a)') buf1
      call trim_c(buf1, nc_buf1)
      call upstr(buf1)
      if (buf1(1:1) == 'Y') then
         susp_log = .true.
      else
         susp_log = .false.
      end if

! Get velocity of first layer:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) vel1
      
!Depth to bottom (d2b) or thickness (thk)?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      type_of_depth = ' '
      read(nu_ctl, '(a)') type_of_depth
      call trim_c(type_of_depth, nc_type_of_depth)
      call upstr(type_of_depth)

! Lines to skip (<0 means use skpcmnts; 0 means skip none):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) nskip

! Ncolumns, Column number for depth/thickness and velocity?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) ncol, icol_dh, icol_vel

!Specify units of velocity (kps, mps):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      units_c = ' '
      read(nu_ctl,'(a)') units_c
      call trim_c(units_c, nc_units_c)
      call upstr(units_c)
      if (units_c(1:3) /= 'MPS' .and. units_c(1:3) /= 'KPS') then
        print *, ' ERROR: units_c(1:3) >= MPS or KPS; it equals '//
     :           units_c(1:3)//' STOP!!!'
        stop
      end if
      
! Get type of depth specification
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) idepth_spec
      
!DEBUG
      write(*,'(a,1x,i3)') ' idepth_spec = ', idepth_spec
!DEBUG
 
! Get depths (ndepths, depths)
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      ntt_divisions = 0
      if (idepth_spec == 1 .or. idepth_spec == 2) then
        read(nu_ctl, *) ndepths, depth_start, depth_stop
        if (idepth_spec == 1) then
          if (depth_start == 0) then
            write(*,'(a)') ' ERROR: depths_start = 0; QUITTING'
            stop
          end if
          do idepth = 1, ndepths
            alogD = alog10(depth_start) + 
     :              alog10(depth_stop/depth_start)*
     :              real(idepth - 1)/real(ndepths - 1)
            depth_out(idepth) = 10.0**alogD
          end do
        else
          do idepth = 1, ndepths
            if (depth_start == 0.0) then
              depth_out(idepth) =  
     :              (depth_stop)*
     :              real(idepth)/real(ndepths)
            else
              depth_out(idepth) = depth_start + 
     :              (depth_stop - depth_start)*
     :              real(idepth - 1)/real(ndepths - 1)
            end if
          end do
        end if
      else if (idepth_spec == 3) then
        read(nu_ctl, *) ndepths, depth_out(1:ndepths)
      else if (idepth_spec == 4) then
        read(nu_ctl, *) ntt_divisions, n_tt4interp
!DEBUG
      write(*,'(a,1x,i4, 1x,i5)') 
     :        ' ntt_divisions, n_tt4interp =', 
     :          ntt_divisions, n_tt4interp
!DEBUG
      else
        write(*, '(a)') 
     :   ' type of depth specification not 1, 2, or 3; QUITTING'
        stop
      end if
      
      
 
! Read and process files:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

      loop over files: DO

        buf1 = ' '
        read(nu_ctl,'(a)',iostat=status) buf1
        if(status /= 0) then
          EXIT loop over files
        end if
        buf2 = ' '
        read(nu_ctl,'(a)',iostat=status) buf2
        if(status /= 0) then
          EXIT loop over files
        end if
        call trim_c(buf1, nc_buf1)
        call trim_c(buf2, nc_buf2)
        buf4 = ' '
        buf4 = buf1(1:4)
        call upstr(buf4)
        if(nc_buf1 == 0 .or. buf1(1:1) == '!') then
          CYCLE loop over files
        end if
        if(buf4(1:4) == 'STOP') then
          EXIT loop over files
        end if
        
        f_in = ' '
        f_in = buf1(1:nc_buf1)
        call trim_c(f_in, nc_f_in)
        f_out = ' '
        f_out = buf2(1:nc_buf2)
        call trim_c(f_out, nc_f_out)
! DEBUG
      write(nu_debug,'(a)') 
     :   ' f_in, f_out = '//f_in(1:nc_f_in)//', '//f_out(1:nc_f_out)
! DEBUG

        f_exist = .false.
        inquire(file=f_in(1:nc_f_in), exist=f_exist)
        if (.not. f_exist) then
           write(*,'(a)') 
     :  ' ******* FILE '//f_in(1:nc_f_in)//
     :  ' DOES NOT EXIST ******* '
           write(*,'(a)') ' Skipping to next file'
           CYCLE loop over files
        end if
      
        call get_lun(nu_in)
        open(nu_in,file=f_in(1:nc_f_in),
     :     status='unknown')

        call get_lun(nu_out)
        open(nu_out,file=f_out(1:nc_f_out),
     :     status='unknown')

        if (vel_file_format == 1 ) then  ! site_amp output file format; may want to use CASE later, if allow more formats
          call skip(nu_in, 5)
! Get nlayers:
          buf1 = ' '
          read(nu_in,'(a)') buf1
          call trim_c(buf1, nc_buf1) 
! Search for first occurrence of "=" after column 1
          ncol_start = 1
          do i = ncol_start, nc_buf1
            if(buf1(i:i) == '=') then
              ncol_start = i + 1
              exit
            end if
          end do
! Read nlayers:
          read(buf1(ncol_start:nc_buf1),*) nlay_in
! DEBUG
            write(nu_debug,'(1x,a, 1x, i6)') ' nlay_in = ', nlay_in
! DEBUG
! Get velocity model:
          call skip(nu_in,2)
          d2b_in(0) = 0.0
          do i = 1, nlay_in
            read(nu_in,*) depth_in(i), vel_in(i)
          end do
        
        else   ! not site_amp output format

          if (nskip < 0) then
            call skipcmnt(nu_in, cmnts2skip, nc_cmnts2skip)
          else if (nskip > 0) then
            call skip(nu_in, nskip)
          end if
 
          nlay_in = 0
          depth_in(0) = 0.0
         
          loop over layers: DO
            read(nu_in,*,iostat=status) (dum(i), i = 1, ncol)
         
            if (status /= 0) then
              EXIT loop over layers
            end if
         
            nlay_in = nlay_in + 1
            depth_in(nlay_in) = abs(dum(icol_dh))
            vel_in(nlay_in) = dum(icol_vel)

            write(*,*) depth_in(nlay_in), vel_in(nlay_in)
! DEBUG
            write(nu_debug,*) depth_in(nlay_in), vel_in(nlay_in)
! DEBUG
       
          END DO loop over layers
       
        end if	  ! not site_amp output format
      
        close(nu_in)

! Find out something about the type of velocity model:

        if (depth_in(1) == 0.0) then
          write(*,*)
          write(*, '(a)') 
     :  '   Assuming the model to be PIECEWISE CONTINUOUS;'//
     :    ' if not, stop program using ctl-brk, modify input file,'//
     :    ' and start again. '
          constant_vel = 'N'
        else
          constant_vel = 'Y'
        end if
 

!
!$$$$$$$$$$$$$$$$$  NEEDS WORK !!!!!!!!!! $$$$$$$$$$$$$$$$$$$$$$$$$$
!
! The code below for a suspension log model needs work; I cannot remember what I assume 
! about the model (a typical model is a a sequence of depths and velocities).
!
! If a suspension log, assume that the depth corresponds to the middle of a layer of 
! constant velocity.  Redo the input model as if it is a stack of constant velocity layers.

      IF (susp_log) then

        vel_temp = vel_in
        depth_temp = depth_in
        nlay_temp = nlay_in
        
        if (vel1 /= 0.0) then
          vel_in(1) = vel1
        end if

        depth_in(0) = 0.0
        depth_in(1) = 
     :         depth_temp(1) - 0.5*(depth_temp(2) - depth_temp(1))

        nlay_in = nlay_temp + 1

! Now assume that the depth is the midpoint of each layer
! 

        do i = 1, nlay_temp-1
          depth_in(i+1) = 0.5*(depth_temp(i+1) + depth_temp(i))
          vel_in(i+1) = vel_temp(i)
        end do

! depth of last point is a special case:

!DUMMY VALUE:
        nd4tt = 0
        vel_in(nlay_in) = vel_temp(nlay_temp)
        depth_in(nd4tt) = depth_temp(nlay_temp) + 
     :          0.5*(depth_temp(nlay_temp) - depth_temp(nlay_temp-1))

      END IF
!
!$$$$$$$$$$$$$$$$$  NEEDS WORK !!!!!!!!!! $$$$$$$$$$$$$$$$$$$$$$$$$$



! If constant velocity layers, convert into a velocity-depth function (with
! repeated depths at interfaces) so that it is in the same format as a 
! piecewise continuous function.

        IF (constant_vel == 'Y') then
          nlayers = nlay_in
          idepth = 0
          d2b(0) = 0.0
          DO ilayer = 1, nlayers
! first fill in depth-to-bottom array:
            if (type_of_depth(1:nc_type_of_depth) == 'D2B') then
              d2b(ilayer) = depth_in(ilayer)
            else if (type_of_depth(1:nc_type_of_depth) == 'THK') then
              d2b(ilayer) = d2b(ilayer-1) + depth_in(ilayer)
            else
              print *,' type_of_depth = '//
     :                type_of_depth(1:nc_type_of_depth)
              print *,' which is not a legal value; QUITTING'
              close(nu_sum)
              stop
            end if
          
! now assign array values for the interfaces above and below each layer:
!   first above:
            idepth = idepth + 1
            depth_model(idepth) = d2b(ilayer-1)
            vel_model(idepth) = vel_in(ilayer)

!   now below:
            idepth = idepth + 1
            depth_model(idepth) = d2b(ilayer)
            vel_model(idepth) = vel_in(ilayer)
          END DO  ! end loop over layers
          ndepths_model = idepth
        ELSE                 ! constant_vel /= 'Y'
          ndepths_model = nlay_in
          do idepth = 1, ndepths_model
            depth_model(idepth) = depth_in(idepth)
            vel_model(idepth) = vel_in(idepth)
          end do  ! end loop over layers
        END IF  ! End of constant velocity or not

! NOW THE MODEL IS IN A STANDARD FORMAT AND WE CAN TREAT IT IN THE
! SAME WAY, REGARDLESS IF IT A STACK OF CONSTANT VELOCITY LAYERS OR A MIXED SET
! OF LINEAR GRADIENTS AND CONSTANT THICKNESS LAYERS.

! Compute max thickness of input as a way of capturing missing data:
!      thk_max = 0.0
      thk_max = depth_model(1)   ! to catch the first point being at some depth
      do i = 2, ndepths_model
        thk = depth_model(i) - depth_model(i-1)
        if (thk > thk_max) thk_max = thk
      end do

!##############################
! Compute travel times at the model depths
        tt_model(1) = 0.0
        avgvel(1) = vel_model(1)
        DO id = 2, ndepths_model
          del_z = depth_model(id)-depth_model(id-1)
          if (vel_model(id) == vel_model(id-1)) then   ! constant velocity
            slope_v(id-1) = 0.0 
            tt_model(id) = tt_model(id-1) + del_z/vel_model(id)
          else
            if(depth_model(id) == depth_model(id-1)) then  ! step change
              slope_v(id-1) = 1.0e10
              tt_model(id) = tt_model(id-1)
            else
              slope_v(id-1) = (vel_model(id)-vel_model(id-1))/ del_z
              tt_model(id) = tt_model(id-1) + 
     :          (1.0/slope_v(id-1))*alog(vel_model(id)/vel_model(id-1))
            end if
          end if
          avgvel(id) = depth_model(id)/tt_model(id)
        END DO
!##############################

!DEBUG
      write(nu_debug,'(a, 1x,i6)') 
     :       ' depth_model, ndepths_model=', ndepths_model
      do i = 1, ndepths_model
        write(nu_debug,'(1xes10.3)') depth_model(i)
      end do
!DEBUG

! Compute depths if they are to be determined by dividing the total travel time into ntt_divisions

      if (ntt_divisions > 0) then
      
! create a vector with many tt
        delta_d4interp = 
     :            depth_model(ndepths_model)/real(n_tt4interp - 1)
        v_halfspace = -999.9
        d4interp(1) = 0.0
        tt4interp(1) = 0.0
        do i4interp = 2, n_tt4interp
          d4interp(i4interp) = 
     :            real(i4interp - 1)* delta_d4interp
          tt4interp(i4interp) = TT_FOR_PIECEWISE_CONTINUOUS_MODEL(
     :         d4interp(i4interp), depth_model, 
     :         vel_model, tt_model, ndepths_model, v_halfspace)
        end do
!DEBUG
      write(nu_debug,'(a)') 
     :       ' i d4interp tt4interp'
      do i = 1, n_tt4interp
        write(nu_debug,'(1x,i6, 1x,es10.3, 1x,es10.3)') 
     :              i, d4interp(i), tt4interp(i)
      end do
!DEBUG
! create a vector with many tt

        tt2bot = tt_model(ndepths_model) 
        delta_tt = tt2bot/real(ntt_divisions)
        do itt = ntt_divisions, 1, -1
          tt4depths(itt) = tt2bot - real(ntt_divisions - itt)*delta_tt
          ynull = -9.9
          call interpolate_1d(tt4interp, d4interp, n_tt4interp, 
     :    ynull, tt4depths(itt), depth_out(itt))
        end do
        ndepths = ntt_divisions

!DEBUG
        write(nu_debug,*)
        write(nu_debug,'(a,1x,i6, 1x,a,1x,es10.3, 1x,a,1x,es10.3)')
     :      ' ntt_divisions=', ntt_divisions, 'tt2bot=', tt2bot, 
     :      'delta_tt=', delta_tt
        write(nu_debug, '(a)') 'i, tt, depth'
        do itt = 1, ntt_divisions
          write(nu_debug, '(1x,i6, 1x,es10.3, 1x,es10.3)')
     :           itt, tt4depths(itt), depth_out(itt) 
        end do
        write(nu_debug,*)
!DEBUG
        
      end if

! Compute and write tt and velocities for specified layers:     

      if(units_c(1:3) == "MPS") then
        write(nu_out,'(10x,a, 10x,a, 7x,a, 6x,a, 9x,a, 7x,a, 
     :              1x,a, 3x,a,
     :              9x,a, 4x,a, 10x,a)') 
     :   'd2b', 'thk', 'vlayer', 'ttlayer', 'tt2d', 'vavg2d',
     :   'slayer(ms/m)', 'savg(ms/m)',
     :   'fqtr', 'sqrt_vavg'
      else
        write(nu_out,'(10x,a, 10x,a, 7x,a, 6x,a, 9x,a, 7x,a, 
     :              1x,a, 3x,a,
     :              9x,a, 4x,a, 10x,a)') 
     :   'd2b', 'thk', 'vlayer', 'ttlayer', 'tt2d', 'vavg2d',
     :   'slayer(s/km)', 'savg(s/km)',
     :   'fqtr', 'sqrt_vavg'
      end if      
      
      tt(0) = 0.0
      d4tt(0) = 0.0

      do i = 1, ndepths

!        if (d4tt(i) > d2b(nlay)) then
!          close(nu_tt)
!          go to 1000
!        end if

        d4tt(i) = depth_out(i)
        
        v_halfspace = -999.9
        
        tt(i) = TT_FOR_PIECEWISE_CONTINUOUS_MODEL(d4tt(i), depth_model, 
     :         vel_model, tt_model, ndepths_model, v_halfspace)

        vlayer = (d4tt(i) - d4tt(i-1))/(tt(i) - tt(i-1))
        vavg = d4tt(i)/tt(i)
        if(units_c(1:3) == "MPS") then
          slayer = 1000.0/vlayer
          savg = 1000.0/vavg
        else
          slayer = 1.0/vlayer
          savg = 1.0/vavg
        end if
        fqtr = 1.0/(4.0*tt(i))
        sqrt_vavg = sqrt(vavg)
        
        write(nu_out,'(11(1x,es12.5))') 
     :                 d4tt(i), d4tt(i) - d4tt(i-1),
     :                 vlayer, tt(i) - tt(i-1),
     :                 tt(i), vavg, 
     :                 slayer, savg,
     :                 fqtr, sqrt_vavg
     
      end do

      close(nu_out)

      END DO loop over files
 
      close(nu_sum)
      close(nu_ctl)
 
      end

      include 'vel2tt_subroutines.for'

!      include '\forprogs\tt4vert.for'
!      include '\forprogs\get_lun.for'
!      include '\forprogs\trim_c.for'
!      include '\forprogs\upstr.for'
!      include '\forprogs\skip.for'
!      include '\forprogs\skipcmnt.for'
!      include '\forprogs\density.for'
!      include '\svel\forprogs\nhrpclss.for'
!      include '\forprogs\locate.for'
!      include '\forprogs\tt_for_piecewise_continuous_model.for'

      